/*
* Copyright (c) 2012,cs2c,安全产品开发部
* All rights reserved.
*
* 文件名称：usercapset.h
* 文件标识：
* 摘    要：动态设置用户权能（适用版本nk32/64位系统）
*
* 当前版本：v1.7.4
* 作    者：xuecheng.lv@cs2c.com.cn
* 完成日期：2013-04-18 1609
* 修改说明：添加头文件，制作动态库，提供外部调用函数接口,修改权能标记字符串定义
*
* 系统默认的权能值：("uid = 0   0xffffffff[1] 0xffffffff[0] CAP_ALL\n")
*			        ("uid = 500 0xffffffff[1] 0xb880ffff[0] <delete 16-22,24-26,30>\n");
*			        ("uid = 501 0xfffffffc[1] 0x7880feff[0] <delete 8,16-22,24-26,31-33>\n");
*
*/

#ifndef _USERCAPSET_H
#define _USERCAPSET_H 1

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <linux/capability.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>
#include <grp.h>
#include <pwd.h>

#define USERCAPSET_DEBUG            0                 //1=OPEN
#define USERCAPSET_LOONGSON         0                 //1=loongson 0=x86/x86_64

#if USERCAPSET_LOONGSON

#define __NR_cap_user_set           4339
#define __NR_capget                 4204

#else

	#ifdef __x86_64__
		#define __NR_capget         125 
		#define __NR_cap_user_set   312
	#elif __i386__
		#define __NR_capget         184
		#define __NR_cap_user_set   349
	#endif
#endif

#define CONF_FILE       "/etc/cap.conf"   		//权能配置文件路径
#define USER_NUM           400              		 //结构体数组：400个用户
#define CAP_NUM          CAP_LAST_CAP               //指令总数
#define CAP_VERSION       2                 		//用户当前权能值数组值

//配置文件结构体定义
struct cap_user_conf_t
{
    int   uid;                                          //用户uid
    __u32 cap[CAP_VERSION];//用户当前的权能值--0x12345678
//    char  cap_user_text_data[768];                      //用户当前的权能值--CAP_CHOWN
};

typedef struct cap_user_conf_t cap_user_conf;
#define CAP_TO_INDEX(x) ((x) >> 5)
#define CAP_TO_MASK(x) (1 << ((x) & 31))

#define cap_raise(c, flag) ((c).cap[CAP_TO_INDEX(flag)] |= CAP_TO_MASK(flag))
#define cap_lower(c, flag) ((c).cap[CAP_TO_INDEX(flag)] &= ~CAP_TO_MASK(flag))
#define cap_raised(c,flag)  ((c).cap[CAP_TO_INDEX(flag)] & CAP_TO_MASK(flag))



#define logw(format, arg...) fprintf(stderr, "user_cap_set: "format, ##arg)
//权能标记字符串
char *cap_strings[CAP_NUM] = {
	"CAP_CHOWN",
	"CAP_DAC_OVERRIDE",
	"CAP_DAC_READ_SEARCH",
	"CAP_FOWNER",
	"CAP_FSETID",
	"CAP_KILL",
	"CAP_SETGID",
	"CAP_SETUID",
	"CAP_SETPCAP",
	"CAP_LINUX_IMMUTABLE",
	"CAP_NET_BIND_SERVICE",
	"CAP_NET_BROADCAST",
	"CAP_NET_ADMIN",
	"CAP_NET_RAW",
	"CAP_IPC_LOCK",
	"CAP_IPC_OWNER",
	"CAP_SYS_MODULE",
	"CAP_SYS_RAWIO",
	"CAP_SYS_CHROOT",
	"CAP_SYS_PTRACE",
	"CAP_SYS_PACCT",
	"CAP_SYS_ADMIN",
	"CAP_SYS_BOOT",
	"CAP_SYS_NICE",
	"CAP_SYS_RESOURCE",
	"CAP_SYS_TIME",
	"CAP_SYS_TTY_CONFIG",
	"CAP_MKNOD",
	"CAP_LEASE",
	"CAP_AUDIT_WRITE",
	"CAP_AUDIT_CONTROL",
	"CAP_SETFCAP",
	"CAP_MAC_OVERRIDE",
	"CAP_MAC_ADMIN",
	"CAP_TRUST_OVERRIDE",
	"CAP_ISIC_READ",
	"CAP_ISIC_WRITE",
	"CAP_ISIC_OVERRIDE",
	"CAP_RBAC_CONFIG",
	"CAP_ISIC_ADMIN",
        };

/************************** 函数声明 begin ********************************/
static void usage();                                               			//打印权能对应信息表 ./usercapset -help
static int set_usr_cap(int uid, __u32 cap_data[3]);                                    //判断一个用户是否有某一能力      
static int filter_cap_record(__u32 *effective, char *record, int uid ,int add_opt);   //解析用户设置的命令
static int load_to_kernel();                                          //开机时将权能信息从配置文件中加载 ./usercapset -k
static int get_current_user_cap();                                    //获取当前用户的权能值(通过系统调用获得，不读文件),并显示 ./usercapset -c
static int del_uid_file(int uid);                                      //根据uid删除配置文件中某一行的数据 ./usercapset -d uid  20120815
/************************** 函数声明 end **********************************/

/************************** 外部函数接口 begin ********************************/

/*
* 查看用户权能列表 usercapset -v
*
* @input: void
* @output:
* @return: 0=ok
*
* 20130418 xuecheng.lv@cs2c.com.cn
*/
extern int usercapset_display();

/*
* 将权能信息从文件中(/etc/cap.conf)加载到内核 usercapset -k
*
* @input: void
* @output:
* @return: 0=ok
*
* 20130418 xuecheng.lv@cs2c.com.cn
*/
extern int usercapset_load_into_kernel();

/*
* 获取当前用户的权能值(通过系统调用获得，不读文件),并显示 ./usercapset -c
*
* @input: void
* @output:
* @return: 0=ok
*
* 20130418 xuecheng.lv@cs2c.com.cn
*/
extern int usercapset_current_user_cap();

/*
* 获取当前用户的权能值(通过系统调用获得，不读文件),并显示 ./usercapset -c
*
* @input: void
* @output:
* @return: 0=ok
*
* 20130418 xuecheng.lv@cs2c.com.cn
*/
//extern int usercapset_current_user_cap1(struct __user_cap_data_struct cap_data_data);

/*
* 设置用户权能
*
* @input:
*        uid:用户uid
*        effective：权能值
*        w：是否写入配置文件(1=写/0=不写) w=1:将当前设置写入配置文件/etc/cap.conf，下次开机自动加载
* @output:
* @return: 0=ok
*
* 20130418 xuecheng.lv@cs2c.com.cn
*/
extern int usercapset_set_cap(int uid, unsigned int effective, int w);
/************************** 外部接口函数 end   ********************************/

#endif/* usercapset.h */
